
#ifndef _DCF77_H    /* Guard against multiple inclusion */
#define _DCF77_H

#include <time.h>
#include "stm32l0xx_hal.h"
#include "main.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif
    
#define ON_RISING_FLANK

#define DCF77_INPUT_PIN HAL_GPIO_ReadPin(DCF77_IN_GPIO_Port, DCF77_IN_Pin)
    
#define MIN_TIME 1334102400UL     // Date: 11-4-2012
#define MAX_TIME 4102444800UL     // Date:  1-1-2100
    
#define SECS_PER_MIN 60
#define SECS_PER_HOUR 3600

#define DCFRejectionTime 700    // Pulse-to-Pulse rejection time. 
#define DCFRejectPulseWidth 50  // Minimal pulse width
#define DCFSplitTime 180        // Specifications distinguishes pulse width 100 ms and 200 ms. In practice we see 130 ms and 230
#define DCFSyncTime 1500        // Specifications defines 2000 ms pulse for end of sequence
    
#ifdef ON_RISING_FLANK
#define PULSE_START 1
#else
#define PULSE_START 0
#endif

// DCF time format structure
typedef struct DCF77Buffer {
  //uint64_t prefix       :21;
  uint64_t prefix     :17;
  uint64_t CEST       :1; // CEST 
  uint64_t CET        :1; // CET 
  uint64_t unused     :2; // unused bits
  uint64_t Min        :7; // minutes
  uint64_t P1         :1; // parity minutes
  uint64_t Hour       :6; // hours
  uint64_t P2         :1; // parity hours
  uint64_t Day        :6; // day
  uint64_t Weekday    :3; // day of week
  uint64_t Month      :5; // month
  uint64_t Year       :8; // year (5 -> 2005)
  uint64_t P3         :1; // parity
} dcf77_buffer_t;

// DCF Parity format structure
typedef struct ParityFlags{
    uint8_t parityFlag    :1;
    uint8_t parityMin     :1;
    uint8_t parityHour    :1;
    uint8_t parityDate    :1;
} dcf77_flags_t;


// Public Functions
void DCF77_Init(void); 
time_t DCF77_getTime(void);
time_t DCF77_getUTCTime(void);
void DCF77_Start(void);
void DCF77_Stop(void);
void DCF77_intHandler(void);
    
#ifdef __cplusplus
}
#endif

#endif /* _DCF77_H */
